/**
 * FwkMap
 * @author wangbinbin
 */

/* ******************* */
/* Constructor & Init  */
/* ******************* */
var FwkMap;

if (FwkMap == undefined) {
	FwkMap = function (settings) {
		return this.initFwkMap(settings);
	};
}

FwkMap.prototype.initFwkMap = function (settings) {
	try {
		this.customSettings = {};	// A container where developers can place their own settings associated with this instance.
		this.settings = settings;
		this.eventQueue = [];
		this.widgetName = "FwkMap_" + FwkMap.widgetCount++;
		this.widgetElement = null;


		// Setup global control tracking
		FwkMap.instances[this.widgetName] = this;

		// Load the settings.  Load the widget.
		this.initSettings();
		this.loadFwkMap();
		this.displayDebugInfo();
		return this;
	} catch (ex) {
		delete FwkMap.instances[this.widgetName];
		throw ex;
		
	}
};

/* *************** */
/* Static Members  */
/* *************** */
FwkMap.instances = {};
FwkMap.widgetCount = 0;
FwkMap.version = "1.0.0 2014-05-21";

FwkMap.ANCHOR = {
		TOP_LEFT 	  		: "BMAP_ANCHOR_TOP_LEFT",
		TOP_RIGHT   		: "BMAP_ANCHOR_TOP_RIGHT",
		BOTTOM_LEFT	  		: "BMAP_ANCHOR_BOTTOM_LEFT",
		BOTTOM_RIGHT   		: "BMAP_ANCHOR_BOTTOM_RIGHT"
	};

/* ******************** */
/* Instance Members  */
/* ******************** */

// Private: initSettings ensures that all the
// settings are set, getting a default value if one was not assigned.
FwkMap.prototype.initSettings = function () {
	this.ensureDefault = function (settingName, defaultValue) {
		this.settings[settingName] = (this.settings[settingName] == undefined) ? defaultValue : this.settings[settingName];
	};
	
	// Upload backend settings
	this.ensureDefault("id", null);
	this.ensureDefault("map_obj", null);//地图对象
	this.ensureDefault("city_name", null);//城市名称
	this.ensureDefault("center_lng", null);//地图中心点经度
	this.ensureDefault("center_lat", null);//地图中心点纬度
	this.ensureDefault("zoom_level", 14);//地图缩放级别
	this.ensureDefault("has_zoom", false);//是否显示缩放控件
	this.ensureDefault("zoom_obj", null);//缩放控件对象
	this.ensureDefault("zoom_anchor", 0);//缩放控件位置
	this.ensureDefault("zoom_offset_x", 0);//缩放控件横向偏移像素值
	this.ensureDefault("zoom_offset_y", 0);//缩放控件竖向偏移像素值
	this.ensureDefault("has_scale", false);//是否显示比例尺控件
	this.ensureDefault("scale_obj", null);//比例尺控件对象
	this.ensureDefault("scale_anchor", 0);//比例尺控件位置
	this.ensureDefault("scale_offset_x", 0);//比例尺控件横向偏移像素值
	this.ensureDefault("scale_offset_y", 0);//比例尺控件竖向偏移像素值
	this.ensureDefault("markers", null);//地图初始化标注点
	this.ensureDefault("marker_icon", null);//地图标注点的自定义图标
	this.ensureDefault("debug", false);//地图标注点的自定义图标

	this.ensureDefault("debug_handler", this.debugMessage);

	this.ensureDefault("custom_settings", {});
	// Other settings
	this.customSettings = this.settings.custom_settings;
	
	delete this.ensureDefault;
};


//Private: loadMap replaces the placeholder element .
FwkMap.prototype.loadFwkMap = function () {
	var targetElement, tempParent;

	// Make sure an element with the ID we are going to use doesn't already exist
	if (document.getElementById(this.widgetName) !== null) {
		throw "ID " + this.widgetName + " is already in use. The Map could not be added";
	}

	// Get the element where we will be placing the widget
	targetElement = document.getElementById(this.settings.id)

	if (targetElement == undefined) {
		throw "Could not find the placeholder element: " + this.settings.id;
	}
	

	this.settings.map_obj = new BMap.Map(this.settings.id);
	//初始化地图控件
	if(this.settings.city_name){
		this.settings.map_obj.centerAndZoom(this.settings.city_name);
	}else{
		this.settings.map_obj.centerAndZoom(new BMap.Point(this.settings.center_lng,this.settings.center_lat), this.settings.zoom_level);
	}
	if(this.settings.has_zoom){
		//添加缩放控件
		
		this.settings.zoom_obj=new BMap.ZoomControl({anchor:this.settings.zoom_anchor,offset: new BMap.Size(this.settings.zoom_offset_x,this.settings.zoom_offset_y)});
		this.settings.map_obj.addControl(this.settings.zoom_obj);
	}
	if(this.settings.has_scale){
		//添加比例尺控件
		this.settings.scale_obj=new BMap.ScaleControl({anchor:this.settings.scale_anchor,offset: new BMap.Size(this.settings.scale_offset_x,this.settings.scale_offset_y)});
		this.settings.map_obj.addControl(this.settings.scale_obj);
	}
	
	//加载初始标注点
	if(this.settings.markers  && (this.settings.markers instanceof Array)){
		for(var i=0;i<this.settings.markers.length;i++){
			if(this.settings.markers[i] && this.settings.markers[i].lng &&  this.settings.markers[i].lat){
				this.addMarker(this.settings.markers[i].lng,this.settings.markers[i].lat,this.settings.markers[i].text);
			}
		}
		
	}

};

/**
 * 添加标注点
 * @param lng 标注点经度
 * @param lat 标注点纬度
 * @param text 标注名称
 * @author:wangbb
 */
FwkMap.prototype.addMarker=function(lng,lat,text){  // 创建图标对象   
    var myIcon =null;
    if(this.fwk_marker_icon){
	    myIcon=new BMap.Icon(this.fwk_marker_icon, 
	        new BMap.Size(14, 23), {
	         // 指定定位位置
	         // 当标注显示在地图上时，其所指向的地理位置距离图标左上角各偏移7像素和25像素。您可以看到在本例中该位置即是图标中央下端的尖角位置
	         anchor: new BMap.Size(7, 25),
	        });
    }
   // 创建标注对象并添加到地图
    var marker =null;
    if(myIcon){
    	marker=new BMap.Marker(new BMap.Point(lng,lat),{icon:myIcon});
    }else{
    	marker=new BMap.Marker(new BMap.Point(lng,lat));
    }
    console.log(text);
    var infoWindow = new BMap.InfoWindow("<div>"+text+"</div>");
    marker.addEventListener("click", function(){this.openInfoWindow(infoWindow);});
    this.settings.map_obj.addOverlay(marker);
};
/**
 * 只添加一个标注点
 * @param lng 标注点经度
 * @param lat 标注点纬度
 * @author:wangbb
 */
FwkMap.prototype.setOneMarker = function(lng,lat,text){  // 创建图标对象  
	this.clearOverlays();
	this.addMarker(lng, lat,text);
};
/**
 * 清除所有覆盖物
 * @author:wangbb
 */
FwkMap.prototype.clearOverlays = function(){  // 创建图标对象  
	this.settings.map_obj.clearOverlays();
};










//Public: Used to remove a FwkMap instance from the page. This method strives to remove
//all references to the widget, and other objects so memory is properly freed.
//Returns true if everything was destroyed. Returns a false if a failure occurs leaving FwkMap in an inconsistant state.
FwkMap.prototype.destroy = function () {
	try {
		


		// Destroy other references
		FwkMap.instances[this.widgetName] = null;
		delete FwkMap.instances[this.widgetName];

		this.widgetElement = null;
		this.settings = null;
		this.customSettings = null;
		this.eventQueue = null;
		this.widgetName = null;
		
		
		return true;
	} catch (ex2) {
		return false;
	}
};


//Public: displayDebugInfo prints out settings and configuration
//information about this FwkMap instance.
//This function (and any references to it) can be deleted when placing
//FwkMap in production.
FwkMap.prototype.displayDebugInfo = function () {
	this.debug(
		[
			"---FwkMap Instance Info---\n",
			"Version: ", FwkMap.version, "\n",
			"Widget Name: ", this.widgetName, "\n",
			"Settings:\n",
			"\t", "id:                       ", this.settings.id, "\n",
			"\t", "center_lng:               ", this.settings.center_lng, "\n",
			"\t", "center_lat:               ", this.settings.center_lat, "\n",
			"\t", "debug:                    ", this.settings.debug.toString(), "\n"
		].join("")
	);
};

/* Note: addSetting and getSetting are no longer used by FwkMap but are included
*/
//Public: (Deprecated) addSetting adds a setting value. If the value given is undefined or null then the default_value is used.
FwkMap.prototype.addSetting = function (name, value, default_value) {
 if (value == undefined) {
     return (this.settings[name] = default_value);
 } else {
     return (this.settings[name] = value);
	}
};

//Public: (Deprecated) getSetting gets a setting. Returns an empty string if the setting was not found.
FwkMap.prototype.getSetting = function (name) {
 if (this.settings[name] != undefined) {
     return this.settings[name];
	}

 return "";
};


/* *******************************
	Widget Event Interfaces
	These functions are used by Widget to trigger the various
	events.
	
	All these functions a Private.
	
	Because the ExternalInterface library is buggy the event calls
	are added to a queue and the queue then executed by a setTimeout.
	This ensures that events are executed in a determinate order and that
	the ExternalInterface bugs are avoided.
******************************* */

FwkMap.prototype.queueEvent = function (handlerName, argumentArray) {
	// Warning: Don't call this.debug inside here or you'll create an infinite loop
	
	if (argumentArray == undefined) {
		argumentArray = [];
	} else if (!(argumentArray instanceof Array)) {
		argumentArray = [argumentArray];
	}
	
	var self = this;
	if (typeof this.settings[handlerName] === "function") {
		// Queue the event
		this.eventQueue.push(function () {
			this.settings[handlerName].apply(this, argumentArray);
		});
		
		// Execute the next queued event
		setTimeout(function () {
			self.executeNextEvent();
		}, 0);
		
	} else if (this.settings[handlerName] !== null) {
		throw "Event handler " + handlerName + " is unknown or is not a function";
	}
};

// Private: Causes the next event in the queue to be executed.  Since events are queued using a setTimeout
// we must queue them in order to garentee that they are executed in order.
FwkMap.prototype.executeNextEvent = function () {
	// Warning: Don't call this.debug inside here or you'll create an infinite loop

	var  f = this.eventQueue ? this.eventQueue.shift() : null;
	if (typeof(f) === "function") {
		f.apply(this);
	}
};


//Private: removes Widget added fuctions to the DOM node to prevent memory leaks in IE.
//This function is called by Widget each time the ExternalInterface functions are created.
FwkMap.prototype.cleanUp = function (widgetElement) {
	// Pro-actively unhook all the Widget functions
	try {
		if (this.widgetElement && typeof(widgetElement.CallFunction) === "unknown") { // We only want to do this in IE
			this.debug("Removing Widget functions hooks (this should only run in IE and should prevent memory leaks)");
			for (var key in widgetElement) {
				try {
					if (typeof(widgetElement[key]) === "function") {
						widgetElement[key] = null;
					}
				} catch (ex) {
				}
			}
		}
	} catch (ex1) {
	
	}


};


/* Called by FwkMap JavaScript and Widget functions when debug is enabled. By default it writes messages to the
   internal debug console.  You can override this event and have messages written where you want. */
FwkMap.prototype.debug = function (message) {
	this.queueEvent("debug_handler", message);
};


/* **********************************
	Debug Console
	The debug console is a self contained, in page location
	for debug message to be sent.  The Debug Console adds
	itself to the body if necessary.

	The console is automatically scrolled as messages appear.
	
	If you are using your own debug handler or when you deploy to production and
	have debug disabled you can remove these functions to reduce the file size
	and complexity.
********************************** */
   
// Private: debugMessage is the default debug_handler.  If you want to print debug messages
// call the debug() function.  When overriding the function your own function should
// check to see if the debug setting is true before outputting debug information.
FwkMap.prototype.debugMessage = function (message) {
	if (this.settings.debug) {
		var exceptionMessage, exceptionValues = [];

		// Check for an exception object and print it nicely
		if (typeof message === "object" && typeof message.name === "string" && typeof message.message === "string") {
			for (var key in message) {
				if (message.hasOwnProperty(key)) {
					exceptionValues.push(key + ": " + message[key]);
				}
			}
			exceptionMessage = exceptionValues.join("\n") || "";
			exceptionValues = exceptionMessage.split("\n");
			exceptionMessage = "EXCEPTION: " + exceptionValues.join("\nEXCEPTION: ");
			FwkMap.Console.writeLine(exceptionMessage);
		} else {
			FwkMap.Console.writeLine(message);
		}
	}
};

FwkMap.Console = {};
FwkMap.Console.writeLine = function (message) {
	var console, documentForm;

	try {
		console = document.getElementById("FwkMap_Console");

		if (!console) {
			documentForm = document.createElement("form");
			document.getElementsByTagName("body")[0].appendChild(documentForm);

			console = document.createElement("textarea");
			console.id = "FwkMap_Console";
			console.style.fontFamily = "monospace";
			console.setAttribute("wrap", "off");
			console.wrap = "off";
			console.style.overflow = "auto";
			console.style.width = "700px";
			console.style.height = "350px";
			console.style.margin = "5px";
			documentForm.appendChild(console);
		}

		console.value += message + "\n";

		console.scrollTop = console.scrollHeight - console.clientHeight;
	} catch (ex) {
		alert("Exception: " + ex.name + " Message: " + ex.message);
	}
};



